@@ -7,26 +7,23 @@ sub belongs_to {
   my ($class, $rel, $f_class, $cond, $attrs) = @_;
   eval "require $f_class";
   if ($@) {
-    $class->throw($@) unless $@ =~ /Can't locate/;
+    $class->throw_exception($@) unless $@ =~ /Can't locate/;
   }
-
-  my %f_primaries;
-  $f_primaries{$_} = 1 for eval { $f_class->primary_columns };
-  my $f_loaded = !$@;
   
-  # single key relationship
+  # no join condition or just a column name
   if (!ref $cond) {
-    $class->throw("Can't infer join condition for ${rel} on ${class}; unable to load ${f_class}")
-      unless $f_loaded;
+    my %f_primaries = map { $_ => 1 } eval { $f_class->primary_columns };
+    $class->throw_exception("Can't infer join condition for ${rel} on ${class}; unable to load ${f_class}")
+      if $@;
 
     my ($pri, $too_many) = keys %f_primaries;
-    $class->throw("Can't infer join condition for ${rel} on ${class}; ${f_class} has no primary keys")
+    $class->throw_exception("Can't infer join condition for ${rel} on ${class}; ${f_class} has no primary keys")
       unless defined $pri;      
-    $class->throw("Can't infer join condition for ${rel} on ${class}; ${f_class} has multiple primary key")
+    $class->throw_exception("Can't infer join condition for ${rel} on ${class}; ${f_class} has multiple primary keys")
       if $too_many;      
 
     my $fk = defined $cond ? $cond : $rel;
-    $class->throw("Can't infer join condition for ${rel} on ${class}; $fk is not a column")
+    $class->throw_exception("Can't infer join condition for ${rel} on ${class}; $fk is not a column")
       unless $class->has_column($fk);
 
     my $acc_type = $class->has_column($rel) ? 'filter' : 'single';
@@ -35,7 +32,7 @@ sub belongs_to {
       { accessor => $acc_type, %{$attrs || {}} }
     );
   }
-  # multiple key relationship
+  # explicit join condition
   elsif (ref $cond eq 'HASH') {
     my $cond_rel;
     for (keys %$cond) {
@@ -45,14 +42,14 @@ sub belongs_to {
       }
       $cond_rel->{"foreign.$_"} = "self.".$cond->{$_};
     }
-    my $acc_type = (scalar keys %$cond_rel == 1) ? 'filter' : 'single';
+    my $acc_type = (keys %$cond_rel == 1 and $class->has_column($rel)) ? 'filter' : 'single';
     $class->add_relationship($rel, $f_class,
       $cond_rel,
       { accessor => $acc_type, %{$attrs || {}} }
     );
   }
   else {
-    $class->throw('third argument for belongs_to must be undef, a column name, or a join condition');
+    $class->throw_exception('third argument for belongs_to must be undef, a column name, or a join condition');
   }
   return 1;
 }
