@@ -28,6 +28,7 @@ sub parse {
     my ($tr, $data) = @_;
     my $args        = $tr->parser_args;
     my $dbixschema  = $args->{'DBIx::Schema'} || $data;
+    $dbixschema   ||= $args->{'package'};
     
     die 'No DBIx::Schema' unless ($dbixschema);
     if (!ref $dbixschema) {
@@ -71,12 +72,10 @@ sub parse {
         }
         $table->primary_key($source->primary_columns);
 
-
         my @rels = $source->relationships();
         foreach my $rel (@rels)
         {
             my $rel_info = $source->relationship_info($rel);
-            print "Accessor: $rel_info->{attrs}{accessor}\n";
             next if(!exists $rel_info->{attrs}{accessor} ||
                     $rel_info->{attrs}{accessor} eq 'multi');
             # Going by the accessor type isn't such a good idea (yes, I know
@@ -89,18 +88,19 @@ sub parse {
             # for testing is
             # $schema->storage->dbh->do($_) for split(";\n", $sql);
             #         -- mst (03:42 local time, please excuse any mistakes)
-            my $rel_table = $rel_info->{class}->table();
+            my $rel_table = $source->related_source($rel)->name;
             my $cond = (keys (%{$rel_info->{cond}}))[0];
             my ($refkey) = $cond =~ /^\w+\.(\w+)$/;
+            my ($key) = $rel_info->{cond}->{$cond} =~ /^\w+\.(\w+)$/;
             if($rel_table && $refkey)
             { 
                 $table->add_constraint(
                             type             => 'foreign_key', 
-                            name             => "fk_${rel}_id",
-                            fields           => $rel,
+                            name             => "fk_${key}",
+                            fields           => $key,
                             reference_fields => $refkey,
                             reference_table  => $rel_table,
-                                       );
+                );
             }
         }
     }
