@@ -16,6 +16,7 @@ __PACKAGE__->load_classes(qw/
     OneKey
     #dummy
     TwoKeys
+    Serialized
   /]},
   (
     'FourKeys',
@@ -25,7 +26,7 @@ __PACKAGE__->load_classes(qw/
     'Producer',
     'CD_to_Producer',
   ),
-  qw/SelfRefAlias/
+  qw/SelfRefAlias TreeLike/
 );
 
 1;
